@@ -1,10 +1,11 @@
 /*
- * remctl PECL extension for PHP
+ * remctl PECL extension for PHP 7 and later
  *
  * Provides bindings for PHP very similar to the libremctl library for C or
  * the Net::Remctl bindings for Perl.
  *
  * Originally written by Andrew Mortensen <admorten@umich.edu>, 2008
+ * Copyirght 2016 Russ Allbery <eagle@eyrie.org>
  * Copyright 2008 Andrew Mortensen <admorten@umich.edu>
  * Copyright 2008, 2011, 2012, 2014
  *     The Board of Trustees of the Leland Stanford Junior University
@@ -65,7 +66,7 @@ ZEND_GET_MODULE(remctl)
  * Destructor for a remctl object.  Close the underlying connection.
  */
 static void
-php_remctl_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
+php_remctl_dtor(zend_resource *rsrc TSRMLS_DC)
 {
     struct remctl *r = (struct remctl *) rsrc->ptr;
 
@@ -93,14 +94,13 @@ PHP_MINIT_FUNCTION(remctl)
  */
 ZEND_FUNCTION(remctl)
 {
-    zval *cmd_array, **data;
-    void **data_ref;
+    zval *cmd_array, *data;
     HashTable *hash;
-    HashPosition pos;
     char *host, *principal = NULL;
     const char **command = NULL;
     long port;
-    int hlen, plen, count, i, status;
+    size_t hlen, plen;
+    int count, i, status;
     int success = 0;
     struct remctl_result *result = NULL;
 
@@ -139,10 +139,8 @@ ZEND_FUNCTION(remctl)
         RETURN_NULL();
     }
     i = 0;
-    zend_hash_internal_pointer_reset_ex(hash, &pos);
-    data_ref = (void **) &data;
-    while (zend_hash_get_current_data_ex(hash, data_ref, &pos) == SUCCESS) {
-        if (Z_TYPE_PP(data) != IS_STRING) {
+    ZEND_HASH_FOREACH_VAL(hash, data) {
+        if (Z_TYPE_P(data) != IS_STRING) {
             zend_error(E_WARNING, "remctl: command contains non-string\n");
             goto cleanup;
         }
@@ -150,15 +148,14 @@ ZEND_FUNCTION(remctl)
             zend_error(E_WARNING, "remctl: internal error: incorrect count\n");
             goto cleanup;
         }
-        command[i] = estrndup(Z_STRVAL_PP(data), Z_STRLEN_PP(data));
+        command[i] = estrndup(Z_STRVAL_P(data), Z_STRLEN_P(data));
         if (command[i] == NULL) {
             zend_error(E_WARNING, "remctl: estrndup failed\n");
             count = i;
             goto cleanup;
         }
         i++;
-        zend_hash_move_forward_ex(hash, &pos);
-    }
+    } ZEND_HASH_FOREACH_END();
     command[count] = NULL;
 
     /* Run the actual remctl call. */
@@ -177,14 +174,14 @@ ZEND_FUNCTION(remctl)
         goto cleanup;
     }
     if (result->error == NULL)
-        add_property_string(return_value, "error", "", 1);
+        add_property_string(return_value, "error", "");
     else
-        add_property_string(return_value, "error", result->error, 1);
+        add_property_string(return_value, "error", result->error);
     add_property_stringl(return_value, "stdout", result->stdout_buf,
-        result->stdout_len, 1);
+        result->stdout_len);
     add_property_long(return_value, "stdout_len", result->stdout_len);
     add_property_stringl(return_value, "stderr", result->stderr_buf,
-        result->stderr_len, 1);
+        result->stderr_len);
     add_property_long(return_value, "stderr_len", result->stderr_len);
     add_property_long(return_value, "status", result->status);
     success = 1;
@@ -214,7 +211,7 @@ ZEND_FUNCTION(remctl_new)
         zend_error(E_WARNING, "remctl_new: %s", strerror(errno));
         RETURN_NULL();
     }
-    ZEND_REGISTER_RESOURCE(return_value, r, le_remctl_internal);
+    RETURN_RES(zend_register_resource(r, le_remctl_internal));
 }
 
 
@@ -226,7 +223,8 @@ ZEND_FUNCTION(remctl_set_ccache)
     struct remctl *r;
     zval *zrem;
     char *ccache;
-    int clen, status;
+    size_t clen;
+    int status;
 
     status = zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &zrem,
                  &ccache, &clen);
@@ -234,8 +232,8 @@ ZEND_FUNCTION(remctl_set_ccache)
         zend_error(E_WARNING, "remctl_set_ccache: invalid parameters\n");
         RETURN_FALSE;
     }
-    ZEND_FETCH_RESOURCE(r, struct remctl *, &zrem, -1, PHP_REMCTL_RES_NAME,
-        le_remctl_internal);
+    r = zend_fetch_resource(Z_RES_P(zrem), PHP_REMCTL_RES_NAME,
+                            le_remctl_internal);
     if (!remctl_set_ccache(r, ccache))
         RETURN_FALSE;
     RETURN_TRUE;
@@ -250,7 +248,8 @@ ZEND_FUNCTION(remctl_set_source_ip)
     struct remctl *r;
     zval *zrem;
     char *source;
-    int slen, status;
+    size_t slen;
+    int status;
 
     status = zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &zrem,
                  &source, &slen);
@@ -258,8 +257,8 @@ ZEND_FUNCTION(remctl_set_source_ip)
         zend_error(E_WARNING, "remctl_set_source_ip: invalid parameters\n");
         RETURN_FALSE;
     }
-    ZEND_FETCH_RESOURCE(r, struct remctl *, &zrem, -1, PHP_REMCTL_RES_NAME,
-        le_remctl_internal);
+    r = zend_fetch_resource(Z_RES_P(zrem), PHP_REMCTL_RES_NAME,
+                            le_remctl_internal);
     if (!remctl_set_source_ip(r, source))
         RETURN_FALSE;
     RETURN_TRUE;
@@ -273,7 +272,7 @@ ZEND_FUNCTION(remctl_set_timeout)
 {
     struct remctl *r;
     zval *zrem;
-    long timeout;
+    zend_long timeout;
     int status;
 
     status = zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &zrem,
@@ -282,8 +281,8 @@ ZEND_FUNCTION(remctl_set_timeout)
         zend_error(E_WARNING, "remctl_set_timeout: invalid parameters\n");
         RETURN_FALSE;
     }
-    ZEND_FETCH_RESOURCE(r, struct remctl *, &zrem, -1, PHP_REMCTL_RES_NAME,
-        le_remctl_internal);
+    r = zend_fetch_resource(Z_RES_P(zrem), PHP_REMCTL_RES_NAME,
+                            le_remctl_internal);
     if (!remctl_set_timeout(r, timeout))
         RETURN_FALSE;
     RETURN_TRUE;
@@ -301,8 +300,9 @@ ZEND_FUNCTION(remctl_open)
     zval *zrem;
     char *host;
     char *principal = NULL;
-    long port = 0;
-    int hlen, plen, status;
+    zend_long port = 0;
+    size_t hlen, plen;
+    int status;
 
     /* Parse and verify arguments. */
     status = zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|ls", &zrem,
@@ -313,8 +313,8 @@ ZEND_FUNCTION(remctl_open)
     }
     if (plen == 0)
         principal = NULL;
-    ZEND_FETCH_RESOURCE(r, struct remctl *, &zrem, -1, PHP_REMCTL_RES_NAME,
-        le_remctl_internal);
+    r = zend_fetch_resource(Z_RES_P(zrem), PHP_REMCTL_RES_NAME,
+                            le_remctl_internal);
 
     /* Now we have all the arguments and can do the real work. */
     if (!remctl_open(r, host, port, principal))
@@ -329,10 +329,8 @@ ZEND_FUNCTION(remctl_open)
 ZEND_FUNCTION(remctl_command)
 {
     struct remctl *r;
-    zval *zrem, *cmd_array, **data;
-    void **data_ref;
+    zval *zrem, *cmd_array, *data;
     HashTable *hash;
-    HashPosition pos;
     struct iovec *cmd_vec = NULL;
     int i, count, status;
     int success = 0;
@@ -344,8 +342,8 @@ ZEND_FUNCTION(remctl_command)
         zend_error(E_WARNING, "remctl_command: invalid parameters\n");
         RETURN_FALSE;
     }
-    ZEND_FETCH_RESOURCE(r, struct remctl *, &zrem, -1, PHP_REMCTL_RES_NAME,
-        le_remctl_internal);
+    r = zend_fetch_resource(Z_RES_P(zrem), PHP_REMCTL_RES_NAME,
+                            le_remctl_internal);
     hash = Z_ARRVAL_P(cmd_array);
     count = zend_hash_num_elements(hash);
     if (count < 1) {
@@ -364,10 +362,8 @@ ZEND_FUNCTION(remctl_command)
         RETURN_FALSE;
     }
     i = 0;
-    zend_hash_internal_pointer_reset_ex(hash, &pos);
-    data_ref = (void **) &data;
-    while (zend_hash_get_current_data_ex(hash, data_ref, &pos) == SUCCESS) {
-        if (Z_TYPE_PP(data) != IS_STRING) {
+    ZEND_HASH_FOREACH_VAL(hash, data) {
+        if (Z_TYPE_P(data) != IS_STRING) {
             zend_error(E_WARNING,
                 "remctl_command: command contains non-string\n");
             goto cleanup;
@@ -377,17 +373,16 @@ ZEND_FUNCTION(remctl_command)
                 "remctl_command: internal error: incorrect count\n");
             goto cleanup;
         }
-        cmd_vec[i].iov_base = emalloc(Z_STRLEN_PP(data) + 1);
+        cmd_vec[i].iov_base = emalloc(Z_STRLEN_P(data) + 1);
         if (cmd_vec[i].iov_base == NULL) {
             zend_error(E_WARNING, "remctl_command: emalloc failed\n");
             count = i;
             goto cleanup;
         }
-        cmd_vec[i].iov_len = Z_STRLEN_PP(data);
-        memcpy(cmd_vec[i].iov_base, Z_STRVAL_PP(data), cmd_vec[i].iov_len);
+        cmd_vec[i].iov_len = Z_STRLEN_P(data);
+        memcpy(cmd_vec[i].iov_base, Z_STRVAL_P(data), cmd_vec[i].iov_len);
         i++;
-        zend_hash_move_forward_ex(hash, &pos);
-    }
+    } ZEND_HASH_FOREACH_END();
 
     /* Finally, we can do the work. */
     if (!remctl_commandv(r, cmd_vec, count))
@@ -422,8 +417,8 @@ ZEND_FUNCTION(remctl_output)
         zend_error(E_WARNING, "remctl_output: invalid parameters\n");
         RETURN_NULL();
     }
-    ZEND_FETCH_RESOURCE(r, struct remctl *, &zrem, -1, PHP_REMCTL_RES_NAME,
-        le_remctl_internal);
+    r = zend_fetch_resource(Z_RES_P(zrem), PHP_REMCTL_RES_NAME,
+                            le_remctl_internal);
 
     /* Get the output token. */
     output = remctl_output(r);
@@ -440,23 +435,23 @@ ZEND_FUNCTION(remctl_output)
     }
     switch (output->type) {
     case REMCTL_OUT_OUTPUT:
-        add_property_string(return_value, "type", "output", 1);
+        add_property_string(return_value, "type", "output");
         add_property_stringl(return_value, "data", output->data,
-            output->length, 1);
+            output->length);
         add_property_long(return_value, "stream", output->stream);
         break;
     case REMCTL_OUT_ERROR:
-        add_property_string(return_value, "type", "error", 1);
+        add_property_string(return_value, "type", "error");
         add_property_stringl(return_value, "data", output->data,
-            output->length, 1);
+            output->length);
         add_property_long(return_value, "error", output->error);
         break;
     case REMCTL_OUT_STATUS:
-        add_property_string(return_value, "type", "status", 1);
+        add_property_string(return_value, "type", "status");
         add_property_long(return_value, "status", output->status);
         break;
     case REMCTL_OUT_DONE:
-        add_property_string(return_value, "type", "done", 1);
+        add_property_string(return_value, "type", "done");
         break;
     }
 }
@@ -476,8 +471,8 @@ ZEND_FUNCTION(remctl_noop)
         zend_error(E_WARNING, "remctl_noop: invalid parameters\n");
         RETURN_FALSE;
     }
-    ZEND_FETCH_RESOURCE(r, struct remctl *, &zrem, -1, PHP_REMCTL_RES_NAME,
-        le_remctl_internal);
+    r = zend_fetch_resource(Z_RES_P(zrem), PHP_REMCTL_RES_NAME,
+                            le_remctl_internal);
     if (!remctl_noop(r))
         RETURN_FALSE;
     RETURN_TRUE;
@@ -500,12 +495,12 @@ ZEND_FUNCTION(remctl_error)
         zend_error(E_WARNING, "remctl_error: invalid parameters\n");
         RETURN_NULL();
     }
-    ZEND_FETCH_RESOURCE(r, struct remctl *, &zrem, -1, PHP_REMCTL_RES_NAME,
-        le_remctl_internal);
+    r = zend_fetch_resource(Z_RES_P(zrem), PHP_REMCTL_RES_NAME,
+                            le_remctl_internal);
 
     /* Do the work. */
     error = remctl_error(r);
-    RETURN_STRING((char *) error, 1);
+    RETURN_STRING((char *) error);
 }
 
 
@@ -515,7 +510,6 @@ ZEND_FUNCTION(remctl_error)
  */
 ZEND_FUNCTION(remctl_close)
 {
-    struct remctl *r;
     zval *zrem;
     int status;
 
@@ -525,10 +519,8 @@ ZEND_FUNCTION(remctl_close)
         zend_error(E_WARNING, "remctl_error: invalid parameters\n");
         RETURN_NULL();
     }
-    ZEND_FETCH_RESOURCE(r, struct remctl *, &zrem, -1, PHP_REMCTL_RES_NAME,
-        le_remctl_internal);
 
     /* This delete invokes php_remctl_dtor, which calls remctl_close. */
-    zend_list_delete(Z_LVAL_P(zrem));
+    zend_list_delete(Z_RES_P(zrem));
     RETURN_TRUE;
 }
