@@ -2,6 +2,7 @@
  * Internal support functions for the remctld daemon.
  *
  * Written by Russ Allbery <eagle@eyrie.org>
+ * Copyright 2016 Dropbox, Inc.
  * Copyright 2015 Russ Allbery <eagle@eyrie.org>
  * Copyright 2006, 2007, 2008, 2009, 2010, 2012, 2014
  *     The Board of Trustees of the Leland Stanford Junior University
@@ -28,6 +29,7 @@ struct evbuffer;
 struct event;
 struct event_base;
 struct iovec;
+struct process;
 
 /*
  * The maximum size of argc passed to the server (4K arguments), and the
@@ -44,9 +46,18 @@ struct iovec;
  */
 #define TIMEOUT (60 * 60)
 
+/*
+ * Normally set by the build system, but don't fail to compile if it's not
+ * defined since it makes the build rules for the test suite irritating.
+ */
+#ifndef PATH_SUDO
+# define PATH_SUDO "sudo"
+#endif
+
 /* Holds the information about a client connection. */
 struct client {
     int fd;                     /* File descriptor of client connection. */
+    int stderr_fd;              /* stderr file descriptor for remctl-shell. */
     char *hostname;             /* Hostname of client (if available). */
     char *ipaddress;            /* IP address of client as a string. */
     int protocol;               /* Protocol version number. */
@@ -57,6 +68,14 @@ struct client {
     time_t expires;             /* Expiration time of GSS-API session. */
     bool keepalive;             /* Whether keep-alive was set. */
     bool fatal;                 /* Whether a fatal error has occurred. */
+
+    /*
+     * Callbacks used by generic server code handle the separate protocols,
+     * set up when the client opens the connection.
+     */
+    void (*setup)(struct process *);
+    bool (*finish)(struct client *, struct evbuffer *, int);
+    bool (*error)(struct client *, enum error_codes, const char *);
 };
 
 /* Holds the configuration for a single command. */
@@ -70,6 +89,7 @@ struct rule {
     unsigned int *logmask;      /* Zero-terminated list of args to mask. */
     long stdin_arg;             /* Arg to pass on stdin, -1 for last. */
     char *user;                 /* Run executable as user. */
+    char *sudo_user;            /* Run executable as user with sudo. */
     uid_t uid;                  /* Run executable with this UID. */
     gid_t gid;                  /* Run executable with this GID. */
     char *summary;              /* Argument that gives a command summary. */
@@ -136,30 +156,42 @@ bool server_config_acl_permit(const struct rule *, const struct client *);
 void server_config_set_gput_file(char *file);
 
 /* Running commands. */
-void server_run_command(struct client *, struct config *, struct iovec **);
+int server_run_command(struct client *, struct config *, struct iovec **);
 
 /* Freeing the command structure. */
 void server_free_command(struct iovec **);
 
 /* Running processes. */
 bool server_process_run(struct process *process);
+void server_handle_io_event(struct bufferevent *, short, void *);
+void server_handle_input_end(struct bufferevent *, void *);
 
-/* Generic protocol functions. */
+/* Generic GSS-API protocol functions. */
 struct client *server_new_client(int fd, gss_cred_id_t creds);
 void server_free_client(struct client *);
 struct iovec **server_parse_command(struct client *, const char *, size_t);
-bool server_send_error(struct client *, enum error_codes, const char *);
 
 /* Protocol v1 functions. */
+void server_v1_command_setup(struct process *);
 bool server_v1_send_output(struct client *, struct evbuffer *, int status);
+bool server_v1_send_error(struct client *, enum error_codes, const char *);
 void server_v1_handle_messages(struct client *, struct config *);
 
 /* Protocol v2 functions. */
-bool server_v2_send_output(struct client *, int stream, struct evbuffer *);
-bool server_v2_send_status(struct client *, int);
+void server_v2_command_setup(struct process *);
+bool server_v2_command_finish(struct client *, struct evbuffer *, int status);
 bool server_v2_send_error(struct client *, enum error_codes, const char *);
 void server_v2_handle_messages(struct client *, struct config *);
 
+/* ssh protocol functions. */
+struct client *server_ssh_new_client(void);
+void server_ssh_free_client(struct client *);
+struct iovec **server_ssh_parse_command(const char *);
+
+/* libevent utility functions. */
+void server_event_log_callback(int, const char *);
+void server_event_fatal_callback(int);
+
 END_DECLS
 
 #endif /* !SERVER_INTERNAL_H */
